(function() {
  const configs = {
    contextMenuId: null,
  };
  const _contextMenu = {
    /**
     * 移除扩展专属的右键菜单
     */
    _removeContextMenu: function() {
      if (!configs.contextMenuId) return;
      chrome.contextMenus.remove(configs.contextMenuId);
      configs.contextMenuId = null;
    },
    /**
     * 创建扩展专属的右键菜单
     */
    init: function() {
      this._removeContextMenu();
      configs.contextMenuId = chrome.contextMenus.create({
        title: 'CSDN 搜索',
        contexts: ['page', 'selection', 'editable', 'link', 'image'],
        documentUrlPatterns: ['http://*/*', 'https://*/*', 'file://*/*'],
        onclick: (info, tab) => {
          chrome.tabs.executeScript(
            tab.id,
            {
              code:
                '(' +
                function(pInfo) {
                  return pInfo.selectionText;
                }.toString() +
                ')(' +
                JSON.stringify(info) +
                ')',
              allFrames: false,
            },
            function(txt) {
              txt = typeof txt === 'object' ? txt[0] : txt;
              chrome.tabs.create(
                {
                  url: 'https://so.csdn.net/so/search/s.do?from=chrome_plugin&q=' + (txt || ''),
                  active: true,
                },
                function() {}
              );
            }
          );
        },
      });
    },
  };
  const _extensionListener = {
    init: function() {
      chrome.runtime.onMessageExternal.addListener(function(request, sender, sendResponse) {
        if (request) {
          if (request.message && request.message == 'installed') {
            sendResponse({ installed: true });
          }
        }
        return true;
      });
      chrome.runtime.onInstalled.addListener(({ reason, previousVersion }) => {
        switch (reason) {
          case 'update':
            setTimeout(() => {
              chrome.browserAction.setBadgeText({ text: '+1' });
              setTimeout(() => {
                chrome.browserAction.setBadgeText({ text: '' });
              }, 1500);
            }, 1500);
            break;
        }
      });
      chrome.webRequest.onBeforeSendHeaders.addListener(
        details => {
          var refererHeaders = details.requestHeaders.find(v => v.name.toLowerCase() === 'referer');

          if (refererHeaders) {
            var isPassport = refererHeaders.value.indexOf('passport.csdn.net') > -1;
            var isPlugin = refererHeaders.value.indexOf('platform=plugin') > -1;
            if (isPassport && isPlugin) {
              try {
                // 尝试关闭打开的CSDN
                chrome.tabs.query({ active: true }, function(tabs) {
                  const tabId = tabs.length ? tabs[0].id : '';
                  chrome.tabs.executeScript(tabId, { code: 'window.close();' }, function() {
                    chrome.tabs.create({ active: true });
                  });
                });
              } catch (e) {
                chrome.tabs.create({ active: true });
              }
            }
          }
        },
        {
          urls: ['https://www.csdn.net/'],
        },
        ['blocking', 'requestHeaders', chrome.webRequest.OnBeforeSendHeadersOptions.EXTRA_HEADERS].filter(Boolean)
      );
      chrome.webRequest.onBeforeSendHeaders.addListener(
        info => {
          const headers = info.requestHeaders;
          const Url = 'chrome-extension://' + chrome.runtime.id;
          for (var i = headers.length - 1; i >= 0; --i) {
            var header = headers[i];
            var isRemoveKey = header.name.toLowerCase() === 'origin' || header.name.toLowerCase() === 'referer';
            if (isRemoveKey && header.value === Url) {
              headers.splice(i, 1); // Remove header
            }
          }
          return { requestHeaders: headers };
        },
        {
          urls: ['https://bizapi.csdn.net/searchplugin/api/v1/*'],
        },
        ['blocking', 'requestHeaders', chrome.webRequest.OnBeforeSendHeadersOptions.EXTRA_HEADERS].filter(Boolean)
      );
    },
  };

  const CSDNPlugin = {
    init: function() {
      _contextMenu.init();
      _extensionListener.init();
    },
  };
  CSDNPlugin.init();
})();
